@@ -29,9 +29,6 @@
 #include "EditorState.h"
 #include "InputMethodState.h"
 #include "PageClientImpl.h"
-#include "WebsiteDataStore.h"
-#include <WebCore/NotImplemented.h>
-#include <WebCore/UserAgent.h>
 
 #if USE(ATK)
 #include <atk/atk.h>
@@ -41,7 +38,6 @@ namespace WebKit {
 
 void WebPageProxy::platformInitialize()
 {
-    notImplemented();
 }
 
 struct wpe_view_backend* WebPageProxy::viewBackend()
@@ -49,16 +45,6 @@ struct wpe_view_backend* WebPageProxy::viewBackend()
     return static_cast<PageClientImpl&>(pageClient()).viewBackend();
 }
 
-String WebPageProxy::userAgentForURL(const URL&)
-{
-    return userAgent();
-}
-
-String WebPageProxy::standardUserAgent(const String& applicationNameForUserAgent)
-{
-    return WebCore::standardUserAgent(applicationNameForUserAgent);
-}
-
 #if USE(ATK)
 void WebPageProxy::bindAccessibilityTree(const String& plugID)
 {
@@ -68,22 +54,6 @@ void WebPageProxy::bindAccessibilityTree(const String& plugID)
 }
 #endif
 
-void WebPageProxy::saveRecentSearches(const String&, const Vector<WebCore::RecentSearch>&)
-{
-    notImplemented();
-}
-
-void WebPageProxy::loadRecentSearches(const String&, CompletionHandler<void(Vector<WebCore::RecentSearch>&&)>&& completionHandler)
-{
-    notImplemented();
-    completionHandler({ });
-}
-
-void WebsiteDataStore::platformRemoveRecentSearches(WallTime)
-{
-    notImplemented();
-}
-
 void WebPageProxy::updateEditorState(const EditorState& editorState)
 {
     m_editorState = editorState;
